from datetime import datetime, timedelta

from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.safestring import mark_safe
from django.views.generic import ListView

from django_tables2 import Table, LinkColumn
from django_tables2.utils import A
from django_tables2.views import SingleTableMixin

from .models import Company, Person, Ticker, Transaction


class AttrAccessor (object):
    def __init__(self, attribute, attributes):
        self._attribute = attribute
        self._attributes = attributes
    def resolve(self, context):
        return {'year': 1000}
        obj = getattr(context, self._attribute)
        print(obj)
        print(self._attributes)
        return dict((a, getattr(obj, a)) for a in self._attributes)


class TransactionTable(Table):
    id = LinkColumn('transaction', args=[A('pk')])
    person = LinkColumn('person', args=[A('person.pk')])
    date = LinkColumn(
        'date', kwargs=dict((x, A('date.{}'.format(x)))
                            for x in ['year', 'month', 'day']))
    ticker = LinkColumn('ticker', args=[A('ticker.pk')])

    class Meta:
        model = Transaction
        order_by = '-date'


class TransactionTableView(SingleTableMixin, ListView):
    model = Transaction
    table_class = TransactionTable


def _detail(request, title, transactions):
    table = TransactionTable(transactions, order_by=request.GET.get('sort'))
    table.paginate(page=request.GET.get('page', 1))
    return render_to_response(
        'table.html', {'title': title, 'table': table},
        context_instance=RequestContext(request))

def company(request, pk):
    tickers = Ticker.objects.filter(company__id=pk)
    #tks = [ticker.pk for ticker in tickers]
    return _detail(request, Company.objects.get(pk=pk),
                   Transaction.objects.filter(ticker__in=tickers))

def person(request, pk):
    return _detail(request, Person.objects.get(pk=pk),
                   Transaction.objects.filter(person__id=pk))

def ticker(request, pk):
    ticker = Ticker.objects.get(pk=pk)
    company = ticker.company
    url = reverse('company', args=[company.id])
    title = mark_safe(u'{} (<a href="{}">{}</a>)'.format(
            ticker, url, company))
    return _detail(request, title, Transaction.objects.filter(ticker__id=pk))

def date(request, **kwargs):
    for k,v in kwargs.items():
        kwargs[k] = int(v)
    date = datetime(**kwargs)
    next_date = date.date() + timedelta(days=1)
    return _detail(request, date.date().isoformat(),
                   Transaction.objects.filter(date__range=(date,next_date)))
